@@ -6,29 +6,30 @@
 
 // Own
 #include "MainWindow.h"
+#include "AppColorSchemeChooser.h"
 
 // Qt
-#include <QMouseEvent>
 #include <QMenu>
 #include <QMenuBar>
-#include <QStatusBar>
+#include <QMouseEvent>
+#include <QWindow>
 
 // KDE
 #include <KAcceleratorManager>
 #include <KActionCollection>
 #include <KActionMenu>
+#include <KCrash>
 #include <KIconUtils>
-#include <KShortcutsDialog>
 #include <KLocalizedString>
+#include <KShortcutsDialog>
 #include <KWindowEffects>
 
-
 #include <KMessageBox>
+#include <KNotifyConfigWidget>
 #include <KStandardAction>
 #include <KStandardGuiItem>
 #include <KWindowSystem>
 #include <KXMLGUIFactory>
-#include <KNotifyConfigWidget>
 
 #include <kio_version.h>
 #include <kwindowsystem_version.h>
@@ -60,51 +61,18 @@
 
 using namespace Konsole;
 
-MainWindow::MainWindow() :
-    KXmlGuiWindow(),
-    _viewManager(nullptr),
-    _bookmarkHandler(nullptr),
-    _toggleMenuBarAction(nullptr),
-    _newTabMenuAction(nullptr),
-    _pluggedController(nullptr),
-    _menuBarInitialVisibility(true)
+MainWindow::MainWindow()
+    : KXmlGuiWindow()
+    , _viewManager(nullptr)
+    , _bookmarkHandler(nullptr)
+    , _toggleMenuBarAction(nullptr)
+    , _newTabMenuAction(nullptr)
+    , _pluggedController(nullptr)
 {
-    KSharedConfigPtr konsoleConfig = KSharedConfig::openConfig(QStringLiteral("konsolerc"));
-    KConfigGroup cg = konsoleConfig->group(QStringLiteral("MainWindow"));
-    const bool isGroup = cg.exists();
-    if (isGroup) {
-        const QString stateConfig = cg.readEntry(QStringLiteral("State"));
-
-        // If "stateConfig" is empty then this is the very first run,
-        // i.e. no konsolerc file in $HOME
-        _isSavedUiState = !stateConfig.isEmpty();
-    }
-
-    if (isGroup && !KonsoleSettings::saveGeometryOnExit()) {
-        // If we are not using the global Konsole save geometry on exit,
-        // remove all geometry data from [MainWindow] in Konsolerc, so KWin will
-        // manage it directly
-        QMap<QString, QString> configEntries = cg.entryMap();
-        QMapIterator<QString, QString> i(configEntries);
-
-        while (i.hasNext()) {
-            i.next();
-// After https://bugs.kde.org/show_bug.cgi?id=415150 was fixed in 5.74,
-// the config file keys changed
-#if KIO_VERSION < QT_VERSION_CHECK(5, 75, 0)
-            if (i.key().startsWith(QLatin1String("Width"))
-                || i.key().startsWith(QLatin1String("Height"))
-#else
-            if (i.key().contains(QLatin1String(" Width"))
-                || i.key().contains(QLatin1String(" Height"))
-                || i.key().contains(QLatin1String(" XPosition"))
-                || i.key().contains(QLatin1String(" YPosition"))
-#endif
-            ) {
-                cg.deleteEntry(i.key());
-            }
-        }
-    }
+    // Set the WA_NativeWindow attribute to force the creation of the QWindow.
+    // Without this QWidget::windowHandle() returns 0.
+    // See https://phabricator.kde.org/D23108
+    setAttribute(Qt::WA_NativeWindow);
 
     updateUseTransparency();
 
@@ -114,23 +82,15 @@ MainWindow::MainWindow() :
     // create view manager
     _viewManager = new ViewManager(this, actionCollection());
     connect(_viewManager, &Konsole::ViewManager::empty, this, &QWidget::close);
-    connect(_viewManager, &Konsole::ViewManager::activeViewChanged, this,
-            &Konsole::MainWindow::activeViewChanged);
-    connect(_viewManager, &Konsole::ViewManager::unplugController, this,
-            &Konsole::MainWindow::disconnectController);
-    connect(_viewManager, &Konsole::ViewManager::viewPropertiesChanged,
-            bookmarkHandler(), &Konsole::BookmarkHandler::setViews);
-    connect(_viewManager, &Konsole::ViewManager::blurSettingChanged,
-            this, &Konsole::MainWindow::setBlur);
+    connect(_viewManager, &Konsole::ViewManager::activeViewChanged, this, &Konsole::MainWindow::activeViewChanged);
+    connect(_viewManager, &Konsole::ViewManager::unplugController, this, &Konsole::MainWindow::disconnectController);
+    connect(_viewManager, &Konsole::ViewManager::viewPropertiesChanged, bookmarkHandler(), &Konsole::BookmarkHandler::setViews);
+    connect(_viewManager, &Konsole::ViewManager::blurSettingChanged, this, &Konsole::MainWindow::setBlur);
 
-    connect(_viewManager, &Konsole::ViewManager::updateWindowIcon, this,
-            &Konsole::MainWindow::updateWindowIcon);
-    connect(_viewManager, &Konsole::ViewManager::newViewWithProfileRequest,
-            this, &Konsole::MainWindow::newFromProfile);
-    connect(_viewManager, &Konsole::ViewManager::newViewRequest,
-            this, &Konsole::MainWindow::newTab);
-    connect(_viewManager, &Konsole::ViewManager::terminalsDetached, this,
-            &Konsole::MainWindow::terminalsDetached);
+    connect(_viewManager, &Konsole::ViewManager::updateWindowIcon, this, &Konsole::MainWindow::updateWindowIcon);
+    connect(_viewManager, &Konsole::ViewManager::newViewWithProfileRequest, this, &Konsole::MainWindow::newFromProfile);
+    connect(_viewManager, &Konsole::ViewManager::newViewRequest, this, &Konsole::MainWindow::newTab);
+    connect(_viewManager, &Konsole::ViewManager::terminalsDetached, this, &Konsole::MainWindow::terminalsDetached);
 
     setCentralWidget(_viewManager->widget());
 
@@ -155,14 +115,29 @@ MainWindow::MainWindow() :
 
     // this must come at the end
     applyKonsoleSettings();
-    connect(KonsoleSettings::self(), &Konsole::KonsoleSettings::configChanged, this,
-            &Konsole::MainWindow::applyKonsoleSettings);
+    connect(KonsoleSettings::self(), &Konsole::KonsoleSettings::configChanged, this, &Konsole::MainWindow::applyKonsoleSettings);
 
+    KCrash::initialize();
+}
 
-    // KXMLGui is making the status bar always visible, we need to fix in a proper way.
-    if (statusBar() != nullptr) {
-        statusBar()->installEventFilter(this);
+bool MainWindow::wasWindowGeometrySaved() const
+{
+    KSharedConfigPtr konsoleConfig = KSharedConfig::openConfig(QStringLiteral("konsolerc"));
+    KConfigGroup cg = konsoleConfig->group(QStringLiteral("MainWindow"));
+    if (!cg.exists()) { // First run, no existing konsolerc?
+        return false;
     }
+
+    const QMap<QString, QString> entries = cg.entryMap();
+    for (auto it = entries.cbegin(), itEnd = entries.cend(); it != itEnd; ++it) {
+        const QString configKey = it.key();
+        if (configKey.contains(QLatin1String(" Width")) || configKey.contains(QLatin1String(" Height")) || configKey.contains(QLatin1String(" XPosition"))
+            || configKey.contains(QLatin1String(" YPosition"))) {
+            return true;
+        }
+    }
+
+    return false;
 }
 
 void MainWindow::updateUseTransparency()
@@ -228,12 +203,9 @@ ViewManager *MainWindow::viewManager() const
 
 void MainWindow::disconnectController(SessionController *controller)
 {
-    disconnect(controller, &Konsole::SessionController::titleChanged,
-               this, &Konsole::MainWindow::activeViewTitleChanged);
-    disconnect(controller, &Konsole::SessionController::rawTitleChanged,
-               this, &Konsole::MainWindow::updateWindowCaption);
-    disconnect(controller, &Konsole::SessionController::iconChanged,
-               this, &Konsole::MainWindow::updateWindowIcon);
+    disconnect(controller, &Konsole::SessionController::titleChanged, this, &Konsole::MainWindow::activeViewTitleChanged);
+    disconnect(controller, &Konsole::SessionController::rawTitleChanged, this, &Konsole::MainWindow::updateWindowCaption);
+    disconnect(controller, &Konsole::SessionController::iconChanged, this, &Konsole::MainWindow::updateWindowIcon);
 
     if (auto view = controller->view()) {
         view->removeEventFilter(this);
@@ -260,8 +232,7 @@ void MainWindow::activeViewChanged(SessionController *controller)
     // associate bookmark menu with current session
     bookmarkHandler()->setActiveView(controller);
     disconnect(bookmarkHandler(), &Konsole::BookmarkHandler::openUrl, nullptr, nullptr);
-    connect(bookmarkHandler(), &Konsole::BookmarkHandler::openUrl, controller,
-            &Konsole::SessionController::openUrl);
+    connect(bookmarkHandler(), &Konsole::BookmarkHandler::openUrl, controller, &Konsole::SessionController::openUrl);
 
     if (!_pluggedController.isNull()) {
         disconnectController(_pluggedController);
@@ -274,12 +245,9 @@ void MainWindow::activeViewChanged(SessionController *controller)
     setBlur(ViewManager::profileHasBlurEnabled(SessionManager::instance()->sessionProfile(_pluggedController->session())));
 
     // listen for title changes from the current session
-    connect(controller, &Konsole::SessionController::titleChanged, this,
-            &Konsole::MainWindow::activeViewTitleChanged);
-    connect(controller, &Konsole::SessionController::rawTitleChanged, this,
-            &Konsole::MainWindow::updateWindowCaption);
-    connect(controller, &Konsole::SessionController::iconChanged, this,
-            &Konsole::MainWindow::updateWindowIcon);
+    connect(controller, &Konsole::SessionController::titleChanged, this, &Konsole::MainWindow::activeViewTitleChanged);
+    connect(controller, &Konsole::SessionController::rawTitleChanged, this, &Konsole::MainWindow::updateWindowCaption);
+    connect(controller, &Konsole::SessionController::iconChanged, this, &Konsole::MainWindow::updateWindowIcon);
 
     controller->setShowMenuAction(_toggleMenuBarAction);
     guiFactory()->addClient(controller);
@@ -332,8 +300,7 @@ void MainWindow::setupActions()
     KActionCollection *collection = actionCollection();
 
     // File Menu
-    _newTabMenuAction = new KActionMenu(QIcon::fromTheme(QStringLiteral("tab-new")),
-                                        i18nc("@action:inmenu", "&New Tab"), collection);
+    _newTabMenuAction = new KActionMenu(QIcon::fromTheme(QStringLiteral("tab-new")), i18nc("@action:inmenu", "&New Tab"), collection);
     collection->setDefaultShortcut(_newTabMenuAction, Konsole::ACCEL | Qt::SHIFT | Qt::Key_T);
     collection->setShortcutsConfigurable(_newTabMenuAction, true);
     _newTabMenuAction->setAutoRepeat(false);
@@ -341,7 +308,7 @@ void MainWindow::setupActions()
     collection->addAction(QStringLiteral("new-tab"), _newTabMenuAction);
     collection->setShortcutsConfigurable(_newTabMenuAction, true);
 
-    QAction* menuAction = collection->addAction(QStringLiteral("clone-tab"));
+    QAction *menuAction = collection->addAction(QStringLiteral("clone-tab"));
     menuAction->setIcon(QIcon::fromTheme(QStringLiteral("tab-duplicate")));
     menuAction->setText(i18nc("@action:inmenu", "&Clone Tab"));
     collection->setDefaultShortcut(menuAction, QKeySequence());
@@ -365,12 +332,13 @@ void MainWindow::setupActions()
     KActionMenu *bookmarkMenu = new KActionMenu(i18nc("@title:menu", "&Bookmarks"), collection);
     _bookmarkHandler = new BookmarkHandler(collection, bookmarkMenu->menu(), true, this);
     collection->addAction(QStringLiteral("bookmark"), bookmarkMenu);
-    connect(_bookmarkHandler, &Konsole::BookmarkHandler::openUrls, this,
-            &Konsole::MainWindow::openUrls);
+    connect(_bookmarkHandler, &Konsole::BookmarkHandler::openUrls, this, &Konsole::MainWindow::openUrls);
 
     // Settings Menu
     _toggleMenuBarAction = KStandardAction::showMenubar(menuBar(), &QMenuBar::setVisible, collection);
     collection->setDefaultShortcut(_toggleMenuBarAction, Konsole::ACCEL | Qt::SHIFT | Qt::Key_M);
+    // Set up themes
+    actionCollection()->addAction(QStringLiteral("window-colorscheme-menu"), new AppColorSchemeChooser(actionCollection()));
 
     // Full Screen
     menuAction = KStandardAction::fullScreen(this, &MainWindow::viewFullScreen, this, collection);
@@ -395,14 +363,18 @@ void MainWindow::setupActions()
     action->setEnabled(true);
     action->setText(i18nc("@action:inmenu", "Save tab layout to file"));
     connect(action, &QAction::triggered, this, [this]() {
-        if (viewManager()) { viewManager()->saveLayoutFile(); }
+        if (viewManager()) {
+            viewManager()->saveLayoutFile();
+        }
     });
 
     action = collection->addAction(QStringLiteral("load-layout"));
     action->setEnabled(true);
     action->setText(i18nc("@action:inmenu", "Load tab layout from file"));
     connect(action, &QAction::triggered, this, [this]() {
-        if (viewManager()) { viewManager()->loadLayoutFile(); }
+        if (viewManager()) {
+            viewManager()->loadLayoutFile();
+        }
     });
 }
 
@@ -415,6 +387,18 @@ void MainWindow::viewFullScreen(bool fullScreen)
     }
 }
 
+void MainWindow::applyMainWindowSettings(const KConfigGroup &config)
+{
+    KMainWindow::applyMainWindowSettings(config);
+
+    // Override the menubar state from the config file
+    if (_windowArgsMenuBarVisible.enabled) {
+        menuBar()->setVisible(_windowArgsMenuBarVisible.showMenuBar);
+    }
+
+    _toggleMenuBarAction->setChecked(menuBar()->isVisibleTo(this));
+}
+
 BookmarkHandler *MainWindow::bookmarkHandler() const
 {
     return _bookmarkHandler;
@@ -430,47 +414,24 @@ void MainWindow::setProfileList(ProfileList *list)
 
 void MainWindow::profileListChanged(const QList<QAction *> &sessionActions)
 {
-    // If only 1 profile is to be shown in the menu, only display
-    // it if it is the non-default profile.
-    if (sessionActions.size() > 2) {
-        // Update the 'New Tab' KActionMenu
-        if (_newTabMenuAction->menu() != nullptr) {
-            _newTabMenuAction->menu()->clear();
-        } else {
-            _newTabMenuAction->setMenu(new QMenu());
-        }
-        for (QAction *sessionAction : sessionActions) {
-            _newTabMenuAction->menu()->addAction(sessionAction);
+    // Update the 'New Tab' KActionMenu
+    _newTabMenuAction->menu()->clear();
+    for (QAction *sessionAction : sessionActions) {
+        _newTabMenuAction->menu()->addAction(sessionAction);
 
-            auto setActionFontBold = [sessionAction](bool isBold) {
-                QFont actionFont = sessionAction->font();
-                actionFont.setBold(isBold);
-                sessionAction->setFont(actionFont);
-            };
+        auto setActionFontBold = [sessionAction](bool isBold) {
+            QFont actionFont = sessionAction->font();
+            actionFont.setBold(isBold);
+            sessionAction->setFont(actionFont);
+        };
 
-            Profile::Ptr profile = ProfileManager::instance()->defaultProfile();
-            if (profile && profile->name() == sessionAction->text().remove(QLatin1Char('&'))) {
-                QIcon icon = KIconUtils::addOverlay(QIcon::fromTheme(profile->icon()), QIcon::fromTheme(QStringLiteral("emblem-favorite")), Qt::BottomRightCorner);
-                sessionAction->setIcon(icon);
-                setActionFontBold(true);
-            } else {
-                setActionFontBold(false);
-            }
-        }
-    } else {
-        if (_newTabMenuAction->menu() != nullptr) {
-            _newTabMenuAction->menu()->clear();
-        } else {
-            _newTabMenuAction->setMenu(new QMenu());
-        }
         Profile::Ptr profile = ProfileManager::instance()->defaultProfile();
-
-        // NOTE: Compare names w/o any '&'
-        if (sessionActions.size() == 2
-            && sessionActions[1]->text().remove(QLatin1Char('&')) != profile->name()) {
-            _newTabMenuAction->menu()->addAction(sessionActions[1]);
+        if (profile && profile->name() == sessionAction->text().remove(QLatin1Char('&'))) {
+            QIcon icon = KIconUtils::addOverlay(QIcon::fromTheme(profile->icon()), QIcon::fromTheme(QStringLiteral("emblem-favorite")), Qt::BottomRightCorner);
+            sessionAction->setIcon(icon);
+            setActionFontBold(true);
         } else {
-            _newTabMenuAction->menu()->deleteLater();
+            setActionFontBold(false);
         }
     }
 }
@@ -510,8 +471,7 @@ void MainWindow::newTab()
 void MainWindow::addPlugin(IKonsolePlugin *plugin)
 {
     _plugins.append(plugin);
-    connect(_viewManager, &Konsole::ViewManager::activeViewChanged, plugin,
-        &IKonsolePlugin::activeViewChanged);
+    connect(_viewManager, &Konsole::ViewManager::activeViewChanged, plugin, &IKonsolePlugin::activeViewChanged);
 }
 
 void MainWindow::cloneTab()
@@ -601,8 +561,8 @@ bool MainWindow::queryClose()
     // Check what processes are running, excluding the shell
     QStringList processesRunning;
     // Need to make a local copy so the begin() and end() point to the same QList
-    const QList<Session*> sessionList = _viewManager->sessions();
-    const QSet<Session*> uniqueSessions(sessionList.begin(), sessionList.end());
+    const QList<Session *> sessionList = _viewManager->sessions();
+    const QSet<Session *> uniqueSessions(sessionList.begin(), sessionList.end());
 
     for (Session *session : uniqueSessions) {
         if ((session == nullptr) || !session->isForegroundProcessActive()) {
@@ -642,21 +602,19 @@ bool MainWindow::queryClose()
     if (!processesRunning.isEmpty()) {
         if (openTabs == 1) {
             result = KMessageBox::warningYesNoList(this,
-                                                         i18ncp("@info",
-                                                                "There is a process running in this window. "
-                                                                "Do you still want to quit?",
-                                                                "There are %1 processes running in this window. "
-                                                                "Do you still want to quit?",
-                                                                processesRunning.count()),
-                                                         processesRunning,
-                                                         i18nc("@title", "Confirm Close"),
-                                                         KGuiItem(i18nc("@action:button",
-                                                                        "Close &Window"),
-                                                                  QStringLiteral("window-close")),
-                                                         KStandardGuiItem::cancel(),
-                                                         // don't ask again name is wrong but I can't update.
-                                                         // this is not about tabs anymore. it's about empty tabs *or* splits.
-                                                         QStringLiteral("CloseAllTabs"));
+                                                   i18ncp("@info",
+                                                          "There is a process running in this window. "
+                                                          "Do you still want to quit?",
+                                                          "There are %1 processes running in this window. "
+                                                          "Do you still want to quit?",
+                                                          processesRunning.count()),
+                                                   processesRunning,
+                                                   i18nc("@title", "Confirm Close"),
+                                                   KGuiItem(i18nc("@action:button", "Close &Window"), QStringLiteral("window-close")),
+                                                   KStandardGuiItem::cancel(),
+                                                   // don't ask again name is wrong but I can't update.
+                                                   // this is not about tabs anymore. it's about empty tabs *or* splits.
+                                                   QStringLiteral("CloseAllTabs"));
             if (result == KMessageBox::No) // No is equal to cancel closing
                 result = KMessageBox::Cancel;
         } else {
@@ -669,12 +627,8 @@ bool MainWindow::queryClose()
                                                                 processesRunning.count()),
                                                          processesRunning,
                                                          i18nc("@title", "Confirm Close"),
-                                                         KGuiItem(i18nc("@action:button",
-                                                                        "Close &Window"),
-                                                                  QStringLiteral("window-close")),
-                                                         KGuiItem(i18nc("@action:button",
-                                                                        "Close Current &Tab"),
-                                                                  QStringLiteral("tab-close")),
+                                                         KGuiItem(i18nc("@action:button", "Close &Window"), QStringLiteral("window-close")),
+                                                         KGuiItem(i18nc("@action:button", "Close Current &Tab"), QStringLiteral("tab-close")),
                                                          KStandardGuiItem::cancel(),
                                                          // don't ask again name is wrong but I can't update.
                                                          // this is not about tabs anymore. it's about empty tabs *or* splits.
@@ -687,11 +641,8 @@ bool MainWindow::queryClose()
                                                        "Do you still want to quit?",
                                                        openTabs),
                                                  i18nc("@title", "Confirm Close"),
-                                                 KGuiItem(i18nc("@action:button", "Close &Window"),
-                                                          QStringLiteral("window-close")),
-                                                 KGuiItem(i18nc("@action:button",
-                                                                "Close Current &Tab"),
-                                                          QStringLiteral("tab-close")),
+                                                 KGuiItem(i18nc("@action:button", "Close &Window"), QStringLiteral("window-close")),
+                                                 KGuiItem(i18nc("@action:button", "Close Current &Tab"), QStringLiteral("tab-close")),
                                                  KStandardGuiItem::cancel(),
                                                  // don't ask again name is wrong but I can't update.
                                                  // this is not about tabs anymore. it's about empty tabs *or* splits.
@@ -749,8 +700,7 @@ void MainWindow::syncActiveShortcuts(KActionCollection *dest, const KActionColle
 
 void MainWindow::showShortcutsDialog()
 {
-    KShortcutsDialog dialog(KShortcutsEditor::AllActions,
-                            KShortcutsEditor::LetterShortcutsDisallowed, this);
+    KShortcutsDialog dialog(KShortcutsEditor::AllActions, KShortcutsEditor::LetterShortcutsDisallowed, this);
 
     // add actions from this window and the current session controller
     const QList<KXMLGUIClient *> clientsList = guiFactory()->clients();
@@ -816,9 +766,11 @@ void MainWindow::showSettingsDialog(const bool showProfilePage)
     generalPage->setIcon(QIcon::fromTheme(QStringLiteral("utilities-terminal")));
     confDialog->addPage(generalPage, true);
 
-    auto *profilePage = new KPageWidgetItem(new ProfileSettings(confDialog), profilePageName);
+    auto *profileSettings = new ProfileSettings(confDialog);
+    auto *profilePage = new KPageWidgetItem(profileSettings, profilePageName);
     profilePage->setIcon(QIcon::fromTheme(QStringLiteral("preferences-system-profiles")));
     confDialog->addPage(profilePage, true);
+    connect(confDialog, &QDialog::accepted, profileSettings, &ProfileSettings::slotAccepted);
 
     const QString tabBarPageName = i18nc("@title Preferences page name", "Tab Bar / Splitters");
     auto tabBarPage = new KPageWidgetItem(new TabBarSettings(confDialog), tabBarPageName);
@@ -844,7 +796,6 @@ void MainWindow::showSettingsDialog(const bool showProfilePage)
 
 void MainWindow::applyKonsoleSettings()
 {
-    setMenuBarInitialVisibility(KonsoleSettings::showMenuBarByDefault());
     setRemoveWindowTitleBarAndFrame(KonsoleSettings::removeWindowTitleBarAndFrame());
     if (KonsoleSettings::allowMenuAccelerators()) {
         restoreMenuAccelerators();
@@ -853,9 +804,10 @@ void MainWindow::applyKonsoleSettings()
     }
 
     _viewManager->activeContainer()->setNavigationBehavior(KonsoleSettings::newTabBehavior());
-    if (KonsoleSettings::saveGeometryOnExit() != autoSaveSettings()) {
-        setAutoSaveSettings(QStringLiteral("MainWindow"), KonsoleSettings::saveGeometryOnExit());
-    }
+
+    // Save the toolbar/menu/dockwidget states and the window geometry
+    setAutoSaveSettings();
+
     updateWindowCaption();
 }
 
@@ -898,13 +850,9 @@ void MainWindow::setBlur(bool blur)
     _blurEnabled = blur;
 
     if (!_pluggedController->isKonsolePart()) {
-#if KWINDOWSYSTEM_VERSION < QT_VERSION_CHECK(5,82,0)
+#if KWINDOWSYSTEM_VERSION < QT_VERSION_CHECK(5, 82, 0)
         KWindowEffects::enableBlurBehind(winId(), blur);
 #else
-        // Set the WA_NativeWindow attribute to force the creation of the QWindow.
-        // Without this QWidget::windowHandle() returns 0.
-        // See https://phabricator.kde.org/D23108
-        setAttribute(Qt::WA_NativeWindow);
         if (QWindow *window = windowHandle()) {
             KWindowEffects::enableBlurBehind(window, blur);
         } else {
@@ -914,9 +862,10 @@ void MainWindow::setBlur(bool blur)
     }
 }
 
-void MainWindow::setMenuBarInitialVisibility(bool visible)
+void MainWindow::setMenuBarInitialVisibility(bool showMenuBar)
 {
-    _menuBarInitialVisibility = visible;
+    _windowArgsMenuBarVisible.enabled = true;
+    _windowArgsMenuBarVisible.showMenuBar = showMenuBar;
 }
 
 void MainWindow::setRemoveWindowTitleBarAndFrame(bool frameless)
@@ -927,7 +876,7 @@ void MainWindow::setRemoveWindowTitleBarAndFrame(bool frameless)
     if (!isVisible()) {
         setWindowFlags(newFlags);
 
-    // The window is visible and the setting changed
+        // The window is visible and the setting changed
     } else if (windowFlags().testFlag(Qt::FramelessWindowHint) != frameless) {
         const auto oldGeometry = saveGeometry();
         // This happens for every Konsole window. It depends on
@@ -949,18 +898,19 @@ void MainWindow::showEvent(QShowEvent *event)
     // Apply this code on first show only
     if (_firstShowEvent) {
         _firstShowEvent = false;
-        // the initial visibility of menubar should be applied at this last
-        // moment. Otherwise, the initial visibility will be determined by
-        // what KMainWindow has automatically stored in konsolerc, but not by
-        // what users has explicitly configured .
-        menuBar()->setVisible(_menuBarInitialVisibility);
-        _toggleMenuBarAction->setChecked(_menuBarInitialVisibility);
 
-        if (!_isSavedUiState || !KonsoleSettings::saveGeometryOnExit()) {
+        if (!KonsoleSettings::rememberWindowSize() || !wasWindowGeometrySaved()) {
             // Delay resizing to here, so that the other parts of the UI
             // (ViewManager, TabbedViewContainer, TerminalDisplay ... etc)
             // have been created and TabbedViewContainer::sizeHint() returns
             // a usuable size.
+
+            // Remove the WindowMaximized state to override the Window-Maximized
+            // config key
+            if (QWindow *window = windowHandle()) {
+                window->setWindowStates(window->windowStates() & ~Qt::WindowMaximized);
+            }
+
             resize(sizeHint());
         }
     }
@@ -981,34 +931,28 @@ void MainWindow::triggerAction(const QString &name) const
 bool MainWindow::eventFilter(QObject *obj, QEvent *event)
 {
     if (!_pluggedController.isNull() && obj == _pluggedController->view()) {
-        switch(event->type()) {
-            case QEvent::MouseButtonPress:
-            case QEvent::MouseButtonDblClick:
-            switch(static_cast<QMouseEvent*>(event)->button()) {
-                case Qt::ForwardButton:
-                    triggerAction(QStringLiteral("next-view"));
-                    break;
-                case Qt::BackButton:
-                    triggerAction(QStringLiteral("previous-view"));
-                    break;
-                default: ;
+        switch (event->type()) {
+        case QEvent::MouseButtonPress:
+        case QEvent::MouseButtonDblClick:
+            switch (static_cast<QMouseEvent *>(event)->button()) {
+            case Qt::ForwardButton:
+                triggerAction(QStringLiteral("next-view"));
+                break;
+            case Qt::BackButton:
+                triggerAction(QStringLiteral("previous-view"));
+                break;
+            default:;
             }
-            default: ;
+        default:;
         }
     }
-    if (qobject_cast<QStatusBar*>(obj) != nullptr) {
-        switch(event->type()) {
-            case QEvent::Show: statusBar()->hide(); break;
-            default: return true;
-        }
-        return true;
-    }
+
     return KXmlGuiWindow::eventFilter(obj, event);
 }
 
 bool MainWindow::focusNextPrevChild(bool v)
 {
-    if (qobject_cast<TerminalDisplay*>(focusWidget())) {
+    if (qobject_cast<TerminalDisplay *>(focusWidget())) {
         return false;
     }
 
